import {
    type Component,
    type JSX,
    ParentProps,
    Show,
    useContext,
} from "solid-js";
import { jwtDecode } from "jwt-decode";
import { Navigate } from "@solidjs/router";
import { save_token } from "tauri-plugin-ios-shared-token-api";
import { platform } from "@tauri-apps/plugin-os";
import { Notifications, NotificationsContext } from "./notifications";

export const isTokenValid = (): boolean => {
    const token = localStorage.getItem("access");

    if (token == null) {
        return false;
    }

    try {
        jwtDecode(token);
        return true;
    } catch (err) {
        return false;
    }
};

const WithNotifications: Component<ParentProps> = (props) => {
    const notifications = Notifications();

    return (
        <NotificationsContext.Provider value={notifications}>
            {props.children}
        </NotificationsContext.Provider>
    );
};

export const useNotifications = () => {
    const notifications = useContext(NotificationsContext);
    if (notifications == null) {
        throw new Error("Notifications must be defined when using this hook");
    }

    return notifications;
};

export const ProtectedRoute: Component<ParentProps> = (props) => {
    const isValid = isTokenValid();

    if (isValid) {
        const token = localStorage.getItem("access");
        if (token == null) {
            throw new Error("unreachable");
        }

        if (platform() === "ios") {
            // iOS share extension is a seperate process to the App.
            // Therefore, we need to share our access token somewhere both the App & Share Extension can access
            // This involves App Groups.
            save_token(token)
                .then(() => console.log("Saved token!!!"))
                .catch((e) => console.error(e));
        }
    }

    return (
        <Show when={isValid} fallback={<Navigate href="/login" />}>
            <WithNotifications>{props.children}</WithNotifications>
        </Show>
    );
};
