import { createStore } from "solid-js/store";
import {
    type InferOutput,
    literal,
    pipe,
    safeParse,
    strictObject,
    string,
    union,
    uuid,
} from "valibot";
import { base } from "../network";
import { createContext } from "solid-js";

const processingImagesValidator = strictObject({
    ImageID: pipe(string(), uuid()),
    Status: union([literal("in-process"), literal("complete")]),
});

type NotificationState = {
    ProcessingImages: Record<
        string,
        InferOutput<typeof processingImagesValidator>["Status"]
    >;
};

export const Notifications = () => {
    const [state, setState] = createStore<NotificationState>({
        ProcessingImages: {},
    });

    const access = localStorage.getItem("access");
    if (access == null) {
        throw new Error("Access token not defined");
    }

    const dataEventListener = (e: MessageEvent<unknown>) => {
        const processingImage = safeParse(processingImagesValidator, e.data);
        if (!processingImage.success) {
            console.error("Processing image could not be parsed.", e.data);
            return;
        }

        const { ImageID, Status } = processingImage.output;

        if (ImageID in state.ProcessingImages) {
            const localImageStatus = state.ProcessingImages[ImageID];
            if (localImageStatus !== "in-process" || Status !== "complete") {
                console.error(
                    "Invalid state, an image present must always be in process",
                );
                return;
            }

            setState("ProcessingImages", (images) => {
                delete images[ImageID];

                // TODO: this might not work because it's not a new object.
                return images;
            });
        } else {
            if (Status !== "in-process") {
                console.error(
                    "Invalid state, at this point a new image that isn't present should be in-progress",
                );
                return;
            }

            setState("ProcessingImages", {
                [ImageID]: Status,
            });
        }
    };

    const events = new EventSource(`${base}/notifications?token=${access}`);

    events.addEventListener("data", dataEventListener);

    return {
        state,
    };
};

export const NotificationsContext =
    createContext<ReturnType<typeof Notifications>>();
